<?php 
namespace Concrete\Package\CcOptinFree;

use Loader;
use Zend_Date;

use Concrete\Package\CcOptinFree\CcOptin;

class CcOptinDate
{
	const dbFormat = 'Y-m-d H:i:s';

	/**
	* For backwards compatibility, use this instead of Loader::helper('date').
	*/
	private static $dh;

	public static function __staticConstruct() {
		self::$dh = Loader::helper('date');
	}

	/*
	* Converts a date from a TZ to UTC in correct format for DB insertion.
	*
	* Default TZ is 'app' (APP_TIMEZONE)
	*/
	public static function toMysqlUTC($date = 'now', $fromTZ = 'app') {
		return self::$dh->formatCustom(self::dbFormat, self::$dh->toDateTime($date, $fromTZ), 'UTC');
	}

	/*
	* Converts a Mysql DATETIME stored in UTC to another timezone, default is app. Uses date().
	* It's very important that this date is in the Mysql DATETIME Y-m-d H:i:s format, 
	* basically coming straight from Mysql.
	*
	* If format is empty, formatDateTime(date, false, true, to) is used.
	*/
	public static function fromMysqlUTC($dateTime, $format = null, $toTZ = 'app') {
		if(empty($dateTime)) return null;

		$date = self::fromMysqlToDateTime($dateTime, $toTZ);

		return $format 
			? $date->format($format)
			: self::$dh->formatDateTime($date, false, true, $toTZ);
	}

	/**
	* The date_time helper seems buddy in 5.7, so it must be manually constructed here.
	*/
	public static function translateDateTimeHelper($prefix, $data) {
		return $data[$prefix.'_dt'] . ' ' . $data[$prefix.'_h'] . ':' . $data[$prefix.'_m'] . 
		   ($data[$prefix.'_a'] == 'AM' || $data[$prefix.'_a'] == 'PM'
		   		? ' ' . $data[$prefix.'_a']
		   		: ':00');
	}

	/**
	* The date_time helper uses system time, so it must be added to the other time here.
	*/
	public static function fromMysqlToDateTimeHelper($dateTime, $toTZ = 'app') {
		if(empty($dateTime)) return null;
		return self::fromMysqlToDateTime($dateTime, $toTZ);
	}

	public static function fromMysqlToDateTime($dateTime, $toTZ = 'app') {
		if(empty($dateTime)) return null;
		
		$dateTime = join('T', explode(' ', $dateTime)) . '+00:00';
		return self::$dh->toDateTime($dateTime, $toTZ, 'UTC');
	}
}

CcOptinDate::__staticConstruct();
