<?php 
namespace Concrete\Package\CcOptinFree\Controllers;

use Loader;
use URL;
use Database;

use Concrete\Package\CcOptinFree\CcOptinCampaignTracker;
use Concrete\Package\CcOptinFree\CcOptinDate;


use Concrete\Package\CcOptinFree\Models\CcOptinCampaign;
use Concrete\Package\CcOptinFree\Models\CcOptinTemplate;
use Concrete\Package\CcOptinFree\Models\CcOptinType;
use Concrete\Package\CcOptinFree\Models\CcOptinDesign;

class CcOptinSaveCampaign
{
	private $post;
	private $campaign;
	private $result;

	public static function newTracker() {
		return uniqid();
	}

	public function __construct($campaign, &$result, $post) {
		$this->campaign = $campaign;
		$this->post = $post;
		$this->val = new CcOptinVal($post, $result);
	}

	public function save() {
		// \Doctrine\DBAL\Logging\EchoSQLLogger
		//$logger = new \Doctrine\DBAL\Logging\DebugStack();
		//Database::connection()->getConfiguration()->setSQLLogger($logger);
		$this->campaign_setDates();
		//fb($logger->queries);
	}

	public function setTypeAndDesign() {
		$this->campaign_setTypeAndDesign();
	}

	private function campaign_setDates() {

		$this->campaign_setProperties();
	}

	private function campaign_setProperties() {
		$this->campaign->active = $this->val->test('active', '0|1');
		$this->campaign->deleted = 0;

		$this->campaign->subpages = 0;

		// Truncate the name, in case "Copy of" gets too long
		$this->campaign->name = substr($this->campaign->name, 0, 255);
		$this->campaign->name = $this->val->test('name', FILTER_SANITIZE_STRING, false);

		// Set a campaign tracker.
		if(!$this->campaign->tracker)
			$this->campaign->tracker = self::newTracker();
		
		$this->campaign_setRules();
	}

	private function campaign_setRules() {
		$rules = 0;

		if($this->post['rules']) {
			foreach ($this->post['rules'] as $bitFlag)
				$rules += intval($bitFlag);
		}

		$this->campaign->rules = $rules;
		
		$this->campaign->rule_showAfterVisits = $this->val->test('rule_showAfterVisits', '[1-9]\d*');
		$this->campaign->rule_hideAfterVisits = $this->val->test('rule_hideAfterVisits', '[2-9]\d*');

		if(($rules & 24) == 24 && $this->campaign->rule_showAfterVisits >= $this->campaign->rule_hideAfterVisits) {
			$this->val->error('rule_showAfterVisits');
		}

		$this->campaign_setSubmitActions();
	}

	private function campaign_setSubmitActions() {
		$templateHandle = $this->val->test('templateHandle', '\w+');
		$template = CcOptinTemplate::get($templateHandle);

		if(!$template) {
			$this->val->error('templateHandle');
			return;
		}

		$submitActions = 0;

		if($this->post['submitActions']) {
			foreach ($this->post['submitActions'] as $bitFlag)
				$submitActions += intval($bitFlag);
		}

		// If template doesn't allow custom content, a submit action must be specified.
		if(!$submitActions && !$template->customContent) {
			$this->val->error('submitActions');
		}

		$this->campaign->submitActions = $submitActions;

		$this->campaign->sa_displayMessage = $this->val->test('sa_displayMessage', '*', false);
		$this->campaign->sa_redirect = $this->val->test('sa_redirect', '|\d+');


		$this->campaign_saveAndLinkToPage();
	}

	private function campaign_saveAndLinkToPage() {
		$cID = 1;
		$this->campaign->id = 1;

		if(!$cID) $this->val->error('cID');
		if($this->val->hasErrors()) return;

		// Need to save here to get id for cID connection
		$this->campaign->save();

		$table = CcOptinCampaign::LinkTable;
		$SQL = "REPLACE INTO $table (campaignID, cID) VALUES (?, ?)";
		Loader::db()->execute($SQL, array($this->campaign->id, $cID));

		$this->campaign_setTypeAndDesign();
	}

	private function campaign_setTypeAndDesign() {
		$id = $this->campaign->id;
		$typeClass = intval($this->post['typeClass']);

		$templateHandle = $this->val->test('templateHandle', '\w+');
		$template = CcOptinTemplate::get($templateHandle);

		if(!$typeClass || !$template) {
			$this->val->errorSummary(t("Couldn't save design for campaign, please try again."));
			return;
		}

		$type = CcOptinType::create($typeClass, 'campaignID = ?', array($id));

		if(!$type->id) {
			$type->campaignID = $id;
			$type->save();
		}

		$design = CcOptinDesign::create($template->designClass, 'campaignID = ?', array($id));

		if(!$design->id) $design->campaignID = $id;
		$design->templateHandle = $templateHandle;
		$design->save();

		// Update campaign with type and design
		$this->campaign->typeID = $type->id;
		$this->campaign->typeClass = $typeClass;
		$this->campaign->designID = $design->id;
		$this->campaign->designClass = $template->designClass;

		// Save and we're done.
		$this->campaign->save();
	}

}

// ================================================================================================

class CampaignController extends CcOptinController
{
	private $db;

	public function __construct() {
		parent::__construct(__FILE__, CcOptinSecurity::read);
		$this->db = Loader::db();
	}

	protected function post($post) {
		if(!$this->security->canWrite()) {
			return $this->outputJsonError(t("You don't have write access."));
		}

		$campaign = new CcOptinCampaign();

		if(intval($post['id'])) {
			$campaign->load('id = ?', $post['id']);
		}

		$result = array();

		$saveCampaign = new CcOptinSaveCampaign($campaign, $result, $post);
		$saveCampaign->save();

		if(!$result['error']) {
			if(!$campaign->id) {
				return $this->outputJsonError(
					t("Cannot save campaign. Please try again, or reload the page if the problem persists.")
				);
			}

			// Reset campaign cookie for user so the campaign can be viewed immediately.
			$tracker = new CcOptinCampaignTracker($campaign);
			$tracker->reset();
		}

		$this->outputJson($result['error'] ? $result : array('id' => $campaign->id));
	}

	protected function get($get) {
		$campaign = new CcOptinCampaign();

		// Load campaign with cID and/or id
		$cID = intval($get['cID']);
		$id = intval($get['id']);

		$table = CcOptinCampaign::LinkTable;
		
		if($cID && !$get['new']) {
			$SQL = "SELECT campaignID FROM $table WHERE cID = 1 ORDER BY campaignID ASC";
			$id = $this->db->getOne($SQL);
			$cID = 1;
		}
		else if(!$get['new']) {
			$id = 1;
		}

		if($id)	$campaign->load('id = ?', $id);

		$otherCampaigns = array();

		$this->requireAsset('javascript', 'cc-scrollspy');
		$this->requireAsset('javascript', 'cc-imgpreview');
		$this->requireAsset('javascript', 'cc-timeago');
		$this->requireAsset('javascript', 'bootstrap/tooltip');
		$this->requireAsset('redactor');

		$this->view('edit', array(
			'campaign' => $campaign, 
			'action' => $this->action(), 
			'cID' => $cID,
			'otherCampaigns' => $otherCampaigns,
			'editContentUrl' => URL::to('/cc_optin/content') . '?id='
		));
	}

	protected function get_templates($get) {
		$handle = preg_match('/^\w+$/', $get['handle']) ? $get['handle'] : '';
		$typeClass = intval($get['typeClass']);

		$this->view('templates', array(
			'handle' => $handle,
			'templates' => CcOptinTemplate::listHandles($typeClass))
		);
	}

	protected function post_reset($post) {
		if(!$this->security->canWrite()) {
			return $this->outputJsonError(t("You don't have write access."));
		}

		$campaign = new CcOptinCampaign();

		$campaign->load('id = 1');

		if(!$campaign->id) return $this->outputJsonError(t("No campaign found."));

		$campaign->tracker = CcOptinSaveCampaign::newTracker();

		if($campaign->save())
			$this->outputJsonSuccess(t('Campaign cookie was resetted.'));
		else
			$this->outputJsonError(t('Could not reset campaign cookie. Please try again soon.'));
	}


	/**
	* Update type and content dynamically.
	*/
	protected function post_setdesign($post) {
		if(!$this->security->canWrite()) {
			return $this->outputJsonError(t("You don't have write access."));
		}

		$campaign = new CcOptinCampaign();

		$id = intval($post['id']);
		if($id)	$campaign->load('id = ?', $id);

		if(!$campaign->id) return $this->outputJsonError(t('No campaign found.'));

		$result = array();
		$saveCampaign = new CcOptinSaveCampaign($campaign, $result, $post);
		$saveCampaign->setTypeAndDesign();

		return $result['error'] 
			? $this->outputJsonError(t("No design selected, using previous one.")) 
			: $this->outputJsonSuccess(t('Campaign design updated.'));
	}	
}
