<?php 
namespace Concrete\Package\CcOptinFree\Controllers;

/**
* Form validation, to be used with ajaxform.coffee
*/
class CcOptinVal
{
	protected $testObj;
	protected $_results;
	protected $_validated;

	protected $addToSummary;
	protected $trimStrings;

	public function __construct($testObj, &$results, $addToSummary = false, $trimStrings = true) {
		$this->testObj = $testObj;
		$this->_results =& $results;

		$this->addToSummary = $addToSummary;
		$this->trimStrings = $trimStrings;

		$this->_validated = array();
	}

	/**
	* Test a field for correct value. $nullIfBlank can be a string and will then be used as $errorMessage
	* $for can be a regex (without delimiters for auto-appending), an array for length(min, max) or 
	* an int for a filter value passed to filter_var().
	* 
	* Returns the value, NULL if blank as default, or FALSE if validation failed.
	*/
	public function test($field, $for, $nullIfBlank = true, $errorMessage = '') {
		$testValue = is_array($this->testObj) ? $this->testObj[$field] : $this->testObj->$field;
		if($this->trimStrings && is_string($testValue)) $testValue = trim($testValue);

		$ok = false;

		if(is_string($nullIfBlank)) $errorMessage = $nullIfBlank;

		if(is_int($for)) {
			// Test with filter_var()
			$ok = filter_var($testValue, $for) !== false;
		}
		else if(is_string($for)) {
			if($for == '*') {
				$ok = true;
			} else {
				// Wrap with regexp if first and last char in string doesn't match
				$first = $for[0]; $last = $for[strlen($for)-1];

				// Wrapper must use a non-capturing group to ensure | is handled correctly.
				if(($first != '/' && $last != '/') && ($first != '#' && $last != '#'))
					$for = '/^(?:' . $for . ')$/';

				$ok = preg_match($for, $testValue);
			}

		}
		else if(is_array($for)) {
			// Test string length or number
			$len = is_string($testValue) ? strlen($testValue) : $testValue;
			$ok = $len >= $for[0] && (count($for) < 2 || $len <= $for[1]);
		}
		else
			throw new \Exception("Undefined validation test method for " . $field);

		if(!$ok) {
			if(!$this->addToSummary)
				$this->error($field, $errorMessage);
			else
				$this->addSummary($errorMessage);

			return false;
		}

		return ($this->_validated[$field] = $nullIfBlank === true ? $this->nullIfBlank($testValue) : $testValue);
	}

	public function nullIfBlank($value) {
		return $value === '' ? null : $value;
	}

	public function error($field, $message = '') {
		$array =& $this->_results;

		if(!$array['error']) $array['error'] = array();
		if(!$array['error']['validation']) $array['error']['validation'] = array();

		$array['error']['validation'][$field] = $message;
	}

	public function errorSummary($message) {
		$array =& $this->_results;

		if(!$array['error']) $array['error'] = array();
		$array['error']['summary'] = $message;
	}

	public function addSummary($message) {
		$array =& $this->_results;

		if(!$array['error']) $array['error'] = array();
		if(!is_array($array['error']['summary'])) $array['error']['summary'] = array();

		$array['error']['summary'][] = $message;
	}

	public static function def($value, $default = null) {
		return empty($value) ? $default : $value;
	}

	public function hasErrors() { return array_key_exists('error', $this->_results); }

	public function results() {	return $this->_results; }

	public function validated() { return $this->_validated; }

	/**
	* Apply the tested properties to the properties of an object.
	* 'id' is never applied as default. (when skip == null)
	*/
	public function applyTo($obj, $skip = null) {
		if($this->hasErrors()) return false;
		if(!$skip) $skip = array('id');

        foreach ($this->_validated as $key => $val) {
            if (property_exists($obj, $key) && !in_array($key, $skip)) {
                $obj->$key = $val;
            }
        }

		return true;
	}
}
